//---------------------------------------------------------------------------
// JrDebug demo that demonstrates unit testing
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// System includes
#include <iostream>
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Include the Debugging Library files
//  Note that only our main .cpp needs to include the jrdebug_main.cpp file
//  alternatively, you could add this cpp to your project or makefile.
#include "../../jrdebug_main.cpp"
//---------------------------------------------------------------------------






//---------------------------------------------------------------------------
// You can place your testing code in this #ifdef block to avoid it compiling
//  in when jrdebug code is not compiled, though this is not at all mandatory,
//  it will compile find even when debugging code is not compiled-in.
#ifdef JrDebugging
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// code for the unit tests

// forward declarations since we are nice
void MyUnitTest2();
void MyUnitTest();


class DummyClass {
public:
	static void ClassStaticTest();
};
void DummyClass::ClassStaticTest() { dbprintf("in static class member test"); }
	

void MyUnitTest2()
{
	dbprintf("my unit test that should throw an exception.");
	// two tests, one that shouldnt fail and will abort further testing if it does
	dbtestex(10<100);

	// waste time
	for (int count1=0;count1<9999;++count1)
		{
		for (int count2=0;count2<9999;++count2)
			count2=count2+1;
		}

	// and another test that will fail and generate a message
	dbtestinfo("this comparison should fail",100<10);
	// manual error messages
	dberror("something interesting happened.");
	debugerr << "something happened " << "interesting";
	debugif(1<2)
		dbprintf("a test shows 1<2.");
	// disable something for an example of a test that we *want* to fail
	JrDebugDisable("purposeful fail");
	dberror("you shouldn't see this!");
	JrDebugEnable("purposeful fail");
	// now throw an exception to show how it will be caught by the testing system
	throw "mystrexception";
	// divide by zero exception not catchable in my dev-c++ ?
	//int mynum = 100 / 0;
}

void MyUnitTest()
{
	dbprintf("Inside my first unit test");
	// waste time
	for (int count1=0;count1<19999;++count1)
		{
		for (int count2=0;count2<9999;++count2)
			count2=count2+1;
		}

	// quiet call to another test
	JrDebugQuietCall(DummyClass::ClassStaticTest());

	//debugout << "Invoking subtest..";
	//dbtestrun(DummyClass::ClassStaticTest);
}
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// register the unit tests - this is all you have to do to tell the system about your test functions
// note how this is done in *global* scope
//
JrDebugRegisterTestFunction("Simple.introtest","A Simple Unit Test",MyUnitTest,"simple");
// note how we can register a (static) member class function as a test function
JrDebugRegisterTestFunction("Advanced.myclasstest","Static Class Member Test",DummyClass::ClassStaticTest,"advanced");
// register another
JrDebugRegisterTestFunction("Advanced.anothertest","Another Unit Test",MyUnitTest2,"advanced");
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
#endif
//---------------------------------------------------------------------------










//---------------------------------------------------------------------------
int main(int argc, char *argv[])
{
	// automatically parse commandline options and exit if appropriate
	if (JrDebugParseCommandlineArgc("Demo9",&argc,argv,"-db"))
		return 0;

	// announce
	std::cout << "Demo program, to run unit tests call on commandline with -dbt\n";

	// all done
	return 0;
}
//---------------------------------------------------------------------------
